/* my fileio */

#ifndef BTRON_FILEIO_H
#define BTRON_FILEIO_H

#include <sys/types.h>
#include <basic.h>
#include "BufferedReader.h"
#include "Exception.h"

#ifdef FILEIO_DEFINE
#define GLOBAL 
#else
#define GLOBAL extern 
#endif

typedef enum tag_DESCRIPTOR_TYPE { 
  DT_FILE, DT_STDOUT, DT_STDERR, 
} DESCRIPTOR_TYPE;

typedef enum tag_OPERATION_TYPE {
  OP_READ, OP_WRITE,
} OPERATION_TYPE;

typedef struct tag_FILE {
  DESCRIPTOR_TYPE type;
  OPERATION_TYPE ope;
  W fd;
  BufferedReader* br;
} FILE;

#ifndef SEEK_SET
#define SEEK_SET 0
#endif
#ifndef SEEK_CUR
#define SEEK_CUR 1
#endif
#ifndef SEEK_END
#define SEEK_END 2
#endif

GLOBAL FILE *fopen(const char *path, const char *mode);
GLOBAL FILE *fopen_link(const LINK *link, const char *mode);
GLOBAL int fclose(FILE *stream);
GLOBAL int fflush(FILE *stream);
GLOBAL int fprintf(FILE *stream, const char *format, ...);
GLOBAL size_t fwrite(const void *ptr, size_t size, size_t nmemb,
                     FILE *stream);
GLOBAL int fseek(FILE *stream, long offset, int whence);
GLOBAL int fseeko(FILE *stream, off_t offset, int whence);
GLOBAL long ftell(FILE *stream);
GLOBAL off_t ftello(FILE *stream);
GLOBAL int fgetc(FILE *stream);
GLOBAL int fputc(int c, FILE *stream); // dummy?
GLOBAL size_t fread(void *ptr, size_t size, size_t nmemb, FILE *stream);
GLOBAL int ungetc(int c, FILE *stream);

#ifndef FILEIO_DEFINE
extern FILE *stdout, *stderr;
#endif

#endif // BTRON_FILEIO_H
