// Vobj Selection handling

#include <basic.h>
#include <bstdio.h>
#include "Vobj.h"
#include "VobjSel.h"

W VobjSelection::getCurrentMode() const
{
  if (selection.empty()) {
    return 0;
  }
  else {
    return selection.begin()->second.region.sts;
  }
}

// TODO: äȸΨǽ
void VobjSelection::append(Vobj* vobj)
{
  drawFlicker(0);
  selection.insert(
    std::make_pair(vobj->getVID(), 
                   SelectedItem(vobj, getCurrentMode(), currentZoom)));
  drawFlicker(1);
}

// TODO: äȸΨǽ
void VobjSelection::remove(VID vid)
{
  drawFlicker(0);
  selection.erase(vid);
  drawFlicker(1);
}

void VobjSelection::clear()
{
  drawFlicker(0);
  selection.clear();
}

Vobj* VobjSelection::findVobj(const PNT& p)
{
  SelectionType::reverse_iterator iend = selection.rend();
  for (SelectionType::reverse_iterator i = selection.rbegin();
       i != iend; ++i) {
    const RECT& org = i->second.vobj->getRect();
    if (org.c.left <= p.x && p.x < org.c.right
        && org.c.top <= p.y && p.y < org.c.bottom) {
      return i->second.vobj;
    }
  }
  return 0;
}

void VobjSelection::move(WID wid, W vcode, int dx, int dy, double zoom)
{
  SelectionType::const_iterator iend = selection.end();
  for (SelectionType::iterator i = selection.begin();
       i != iend; ++i) {
    RECT org = i->second.vobj->getRect();
    SEL_RGN* rgn = &(i->second.region);
    adsp_sel(gid, const_cast<SEL_RGN*>(rgn), 0);
    switch (vcode) {
        case V_LTHD:
          org.c.left += dx;
          org.c.top += dy;
          break;
        case V_LBHD:
          org.c.left += dx;
          org.c.bottom += dy;
          break;
        case V_RTHD:
          org.c.right += dx;
          org.c.top += dy;
          break;
        case V_RBHD:
          org.c.right += dx;
          org.c.bottom += dy;
          break;
        default:
          org.c.left += dx;
          org.c.top += dy;
          org.c.right += dx;
          org.c.bottom += dy;
          break;
    }
    i->second.vobj->move(wid, vcode, &org);
    i->second.calcZoom(zoom);
    adsp_sel(gid, const_cast<SEL_RGN*>(rgn), 1);
  }
}

bool VobjSelection::isSelected(VID vid) const
{
  return (selection.find(vid) != selection.end());
}

void VobjSelection::drawFlicker(int mode) const
{
  SelectionType::const_iterator iend = selection.end();
  for (SelectionType::const_iterator i = selection.begin();
       i != iend; ++i) {
    const SEL_RGN* rgn = &(i->second.region);
    adsp_sel(gid, const_cast<SEL_RGN*>(rgn), mode);
  }
}

// O(1)
bool VobjSelection::changeZoom(double zoom)
{
  if (currentZoom == zoom) {
    return false;
  }
  currentZoom = zoom;
  SelectionType::const_iterator iend = selection.end();
  for (SelectionType::iterator i = selection.begin();
       i != iend; ++i) {
    SEL_RGN* rgn = &(i->second.region);
    adsp_sel(gid, rgn, 0);
    i->second.calcZoom(currentZoom);
    adsp_sel(gid, rgn, 1);
  }
  return true;
}

void VobjSelection::mouse_down(WID wid, WEVENT* ev, int page, double zoom, VobjSet* vobjs, Vobj* vobj)
{
  PNT orgpos, firstp, sp, ep;
  firstp = ev->s.pos;
  orgpos.x = (int)(ev->s.pos.x * 100.0 / zoom);
  orgpos.y = (int)(ev->s.pos.y * 100.0 / zoom);
  W vcode;
  VID vid = vobj->getVID();
  GID dgid;
  W cktype;
  RECT vrect, frect;
  bool onHandle;
  bool lock;

  switch (cktype = wchk_dck(ev->s.time)) {
      case W_CLICK:
        if ((ev->s.stat & (ES_LSHFT | ES_RSHFT)) == 0) {
          clear();
          append(vobj);
        }
        else {
          if (isSelected(vobj->getVID())) {
            remove(vobj->getVID());
          }
          else {
            append(vobj);
          }
        }
        break;
      case W_DCLICK:
        gset_ptr(PS_BUSY, NULL, (COLOR)-1, (COLOR)-1);
        if (oexe_apg(vobj->getVID(), 0) > 0) {
          gset_ptr(0x8001, NULL, (COLOR)-1, (COLOR)-1);
        }
        break;
      case W_PRESS:
      case W_QPRESS:
        vcode = ofnd_vob(-vid, orgpos, &vid);
        if (cktype == W_QPRESS) {
          // W_QPRESS ΤȤϡꥵˤϤʤ
          vcode = V_NAME;
        }
        onHandle = (vcode == V_LTHD || vcode == V_RTHD 
                    || vcode == V_LBHD || vcode == V_RBHD);
        if (cktype == W_PRESS) {
          if (!isSelected(vobj->getVID()) || onHandle) {
            clear();
            append(vobj);
          }
        }
        else {
          if (selection.size() > 1 && isSelected(vobj->getVID())) {
            break;
          }
          clear();
          append(vobj);
        }

        gset_ptr((onHandle ? PS_PICK : PS_GRIP), 
                 NULL, (COLOR)-1, (COLOR)-1);
        sp = ev->s.pos;

        dgid = wsta_drg(wid, 0);
        setAll(gid, 0);
        setAll(dgid, 1);
        lock = false;
        gget_vis(gid, &vrect);
        gget_fra(dgid, &frect);
        while (wget_drg(&ep, ev) != EV_BUTUP) {
          if ((sp.x == ep.x) && (sp.y == ep.y)) {
            continue;
          }
          if (vrect.c.left <= ep.x && ep.x < vrect.c.right 
              && vrect.c.top <= ep.y && ep.y < vrect.c.bottom) {
            if (lock) {
              setAll(dgid, 0);
              gset_vis(dgid, vrect);
              gloc_env(dgid, 0);
              lock = false;
            }
          }
          else {
            if (!lock) {
              setAll(dgid, 0);
              gloc_env(dgid, 1);
              gset_vis(dgid, frect);
              lock = true;
            }
          }
          dragZoom(dgid, vcode, ep.x - sp.x, ep.y - sp.y);
          sp = ep;
        }
        setAll(dgid, 0);
        if (lock) {
          gset_vis(dgid, vrect);
          gloc_env(dgid, 0);
          lock = false;
        }
        wend_drg();
        setAll(gid, 1); // to be deleted

        if ((ev->s.stat & (ES_BUT2|ES_CMD)) != 0) {
          // --- ʣ
          if (ev->s.wid == wid) {
            dragZoom(gid, vcode, firstp.x - sp.x, firstp.y - sp.y);
            vobjCopyToPage(vobjs, page);
            move(wid, vcode, (ep.x - firstp.x) * 100.0 / zoom, 
                 (ep.y - firstp.y) * 100.0 / zoom, zoom);
            vobjs->setChanged();
          }
          else {
            pushTmpTray(ev, vobjs, page, false, firstp, zoom);
            dragZoom(gid, vcode, firstp.x - sp.x, firstp.y - sp.y);
          }
        }
        else if (cktype == W_QPRESS) {
          // --- ¿ʣ
          if (ev->s.wid == wid) {
            dragZoom(gid, vcode, firstp.x - sp.x, firstp.y - sp.y);
            bool changed = robjCopyToPage(vobjs, page);
            if (changed) {
              move(wid, vcode, (ep.x - firstp.x) * 100.0 / zoom, 
                   (ep.y - firstp.y) * 100.0 / zoom, zoom);
              vobjs->setChanged();
            }
          }
          else {
            dragZoom(gid, vcode, firstp.x - sp.x, firstp.y - sp.y);
            robjCopyToTray(ev, firstp, zoom);
          }
        }
        else {
          // --- Ȱư
          if (ev->s.wid == wid) {
            move(wid, vcode, (ep.x - firstp.x) * 100.0 / zoom, 
                 (ep.y - firstp.y) * 100.0 / zoom, zoom);
            vobjs->setChanged();
          }
          else {
            bool removed = pushTmpTray(ev, vobjs, page, true, firstp, zoom);
            if (removed) {
              vobjs->setChanged();
            }
            else {
              dragZoom(gid, vcode, firstp.x - sp.x, firstp.y - sp.y);
            }
          }
        }
        wreq_dsp(wid);
        break;
  }
}

void VobjSelection::setAll(GID tgid, int mode)
{
  SelectionType::const_iterator iend = selection.end();
  for (SelectionType::iterator i = selection.begin();
       i != iend; ++i) {
    const SEL_RGN* rgn = &(i->second.region);
    adsp_sel(tgid, const_cast<SEL_RGN*>(rgn), mode);
  }
}

void VobjSelection::dragZoom(GID dgid, W vcode, int dx, int dy)
{
  SelectionType::const_iterator iend = selection.end();
  for (SelectionType::iterator i = selection.begin();
       i != iend; ++i) {
    SEL_RGN* rgn = &(i->second.region);
    RECT* zoomsize = &(rgn->rgn.r);
    adsp_sel(dgid, const_cast<SEL_RGN*>(rgn), 0);
    switch (vcode) {
        case V_LTHD: // 
          zoomsize->c.left += dx;
          zoomsize->c.top += dy;
          break;
        case V_LBHD: // 
          zoomsize->c.left += dx;
          zoomsize->c.bottom += dy;
          break;
        case V_RTHD: // 
          zoomsize->c.right += dx;
          zoomsize->c.top += dy;
          break;
        case V_RBHD: // 
          zoomsize->c.right += dx;
          zoomsize->c.bottom += dy;
          break;
        default: // ư
          zoomsize->c.left += dx;
          zoomsize->c.top += dy;
          zoomsize->c.right += dx;
          zoomsize->c.bottom += dy;
          break;
    }
    adsp_sel(dgid, const_cast<SEL_RGN*>(rgn), 1);
  }
}

bool VobjSelection::pushTmpTray(WEVENT *ev, VobjSet* vobjs, int page, bool remove, const PNT& firstp, double zoom)
{
  std::vector<TRAYREC> trec(selection.size() + 1);
  RECT r;
  W err;

  if (ev->r.wid <= 0) {
    return false;
  }

  // ǡ trec[] ˵ͤ
  int idx = 1;
  bool first = true;
  SelectionType::const_iterator iend = selection.end();
  for (SelectionType::iterator i = selection.begin();
       i != iend; ++i) {
    Vobj* vobj = i->second.vobj;
    const RECT& vrect = vobj->getRect();

    vobj->refreshData();
    trec[idx].id = TR_VOBJ;
    trec[idx].len = sizeof(VLINK) + SIZE_VOBJSEG(vobj->getVseg()->dlen);
    trec[idx].dt = reinterpret_cast<B*>(new UB[trec[idx].len]);
    TR_VOBJREC* vrec = reinterpret_cast<TR_VOBJREC*>(trec[idx].dt);
    vrec->vlnk = *(vobj->getVlink());
    memcpy(&(vrec->vseg), vobj->getVseg(), SIZE_VOBJSEG(vobj->getVseg()->dlen));

    if (first) {
      first = false;
      r = vrect;
    }
    else {
      if (r.c.left > vrect.c.left) 
        r.c.left = vrect.c.left;
      if (r.c.right < vrect.c.right) 
        r.c.right = vrect.c.right;
      if (r.c.top > vrect.c.top) 
        r.c.top = vrect.c.top;
      if (r.c.bottom < vrect.c.bottom) 
        r.c.bottom = vrect.c.bottom;
    }
    idx++;
  }

  FIGSEG head = { r, r, -120, -120, 0 };
  trec[0].id = TS_FIG;
  trec[0].len = sizeof(FIGSEG);
  trec[0].dt = reinterpret_cast<B*>(&head);

  // ȥ졼 push
  err = tset_dat(&(trec[0]), trec.size());
  if (err < 0) {
    // TODO: errpanel(EPNL_TMPTRAY, err);
    remove = false;
    goto exit0;
  }

  // ǡŽ׵
  ev->r.type = EV_REQUEST;
  ev->r.cmd = W_PASTE;
  ev->r.r.p.lefttop = ev->s.pos;
  ev->r.r.c.right -= (firstp.x * 100.0 / zoom) - r.c.left;
  ev->r.r.c.bottom -= (firstp.y * 100.0 / zoom) - r.c.top;
  err = wsnd_evt(ev);
  if (err < 0) {
    tset_dat(NULL, 0);
    remove = false;
    goto exit0;
  }
  err = wwai_rsp(ev, W_PASTE, 60000); // 1m
  if (err == W_ACK) {
    if ((ev->s.pos.x & 0xffff) == 0x8000 
        && (ev->s.pos.y & 0xffff) == 0x8000) {
      remove = false;
    }
  }
  else {
    tset_dat(NULL, 0);
    remove = false;
    goto exit0;
  }

  // ưβȤκ
  if (remove) {
    deleteFromPage(vobjs, page);
  }

  // Žޤ줿ɥ switch
  wswi_wnd(ev->r.wid, NULL);

exit0:
  // β (trec[0]  &head ʤΤǡʤ)
  for (int i = 1; i < trec.size(); i++) {
    delete [] reinterpret_cast<B*>(trec[i].dt);
  }
  return remove;
}

// TODO: ǰ O(n^2) ʤΤǡˤʤ褦ʤľ
void VobjSelection::deleteFromPage(VobjSet* vobjs, int page)
{
  drawFlicker(0);

  SelectionType::const_iterator iend = selection.end();
  for (SelectionType::const_iterator i = selection.begin();
       i != iend; ++i) {
    vobjs->remove(i->second.vobj->getVID(), page);
  }

  selection.clear();
}

// TODO: ŤΤǡselection  auto_ptr ǽľ٤
void VobjSelection::vobjCopyToPage(VobjSet* vobjs, int page)
{
  drawFlicker(0);

  SelectionType oldsel = selection;
  selection.clear();

  SelectionType::iterator iend = oldsel.end();
  for (SelectionType::iterator i = oldsel.begin();
       i != iend; ++i) {
    Vobj *vobj = i->second.vobj->duplicate();
    selection.insert(
      std::make_pair(vobj->getVID(), 
                     SelectedItem(vobj, getCurrentMode(), currentZoom)));
    vobjs->add(vobj, page);
  }

  drawFlicker(1);
}

// TODO: ŤΤǡselection  auto_ptr ǽľ٤(Ʊ)
bool VobjSelection::robjCopyToPage(VobjSet* vobjs, int page)
{
  bool changed = true;

  drawFlicker(0);

  SelectionType oldsel = selection;
  selection.clear();

  SelectionType::iterator iend = oldsel.end();
  for (SelectionType::iterator i = oldsel.begin();
       i != iend; ++i) {
    Vobj *vobj = i->second.vobj->duplicateRobj();
    if (vobj != 0) {
      selection.insert(
        std::make_pair(vobj->getVID(), 
                       SelectedItem(vobj, getCurrentMode(), currentZoom)));
      vobjs->add(vobj, page);
    }
  }

  if (selection.size() <= 0) {
    selection = oldsel;
    changed = false;
  } 

  drawFlicker(1);

  return changed;
}

// VM_NEW 
void VobjSelection::robjCopyByMenu(VobjSet* vobjs, const W* vlist, int page)
{
  selection.clear();

  for (int i = 0; vlist[i] != 0; i += 2) {
    if (vlist[i+1] <= 0) {
      continue;
    }
    Vobj *vobj = new Vobj(vlist[i+1]);
    selection.insert(
      std::make_pair(vobj->getVID(), 
                     SelectedItem(vobj, getCurrentMode(), currentZoom)));
    vobjs->add(vobj, page);
  }
}

void VobjSelection::robjCopyToTray(WEVENT *ev, const PNT& firstp, double zoom)
{
  VobjSelection copied(-1, zoom);
  
  SelectionType::iterator iend = selection.end();
  for (SelectionType::iterator i = selection.begin();
       i != iend; ++i) {
    Vobj *vobj = i->second.vobj->duplicateRobj(ev->s.wid);
    if (vobj != 0) {
      copied.selection.insert(
        std::make_pair(vobj->getVID(), 
                       SelectedItem(vobj, getCurrentMode(), currentZoom)));
    }
  }

  // removed = false ʤΤǡvobjs, page ϥߡ (ɡ)
  copied.pushTmpTray(ev, NULL, -1, false, firstp, zoom);

  iend = copied.selection.end();
  for (SelectionType::iterator i = copied.selection.begin();
       i != iend; ++i) {
    // VobjSet ǴƤʤΤǡɬ
    delete i->second.vobj;
  }
}

UB* VobjSelection::allocVidList() const
{
  UB* buf = new UB[(sizeof(W) + sizeof(RECT)) * selection.size() + sizeof(W)];

  int ctr = 0;
  SelectionType::const_iterator iend = selection.end();
  for (SelectionType::const_iterator i = selection.begin();
       i != iend; ++i) {
    VID vid = i->second.vobj->getVID();
    reinterpret_cast<VID*>(buf)[ctr] = vid;
    ctr++;
  }
  reinterpret_cast<VID*>(buf)[ctr] = 0;

  return buf;
}

// oget_men, oget_vmn ΤΥ⡼ɷ׻
VID VobjSelection::getMenuMode() const
{
  int nitems = selection.size();

  if (nitems <= 0) {
    return -1;
  }
  else if (nitems > 1) {
    return 0;
  }
  else {
    return selection.begin()->second.vobj->getVID();
  }
}

bool VobjSelection::updateVID(VID vid)
{
  SelectionType::iterator i = selection.find(vid);
  if (i == selection.end()) {
    return false;
  }

  drawFlicker(0); // TODO: ȴ

  i->second.vobj->refreshData();
  i->second.calcZoom(currentZoom);

  drawFlicker(1); // TODO: ȴ

  return true;
}
