//========================================================================
//
// XPDFCore.cc
//
// Copyright 2002-2003 Glyph & Cog, LLC
//
//========================================================================

#include <aconf.h>

#ifdef USE_GCC_PRAGMAS
#pragma implementation
#endif

//#include <X11/keysym.h>
//#include <X11/cursorfont.h>
#include <string.h>
#include "gmem.h"
#include "GString.h"
#include "GList.h"
#include "Error.h"
#include "GlobalParams.h"
#include "PDFDoc.h"
#include "Link.h"
#include "ErrorCodes.h"
#include "GfxState.h"
#include "CoreOutputDev.h"
#include "PSOutputDev.h"
#include "TextOutputDev.h"
#include "SplashBitmap.h"
#include "SplashPattern.h"
#include "XPDFApp.h"
#include "XPDFCore.h"

extern DEV_SPEC SCREEN;

// these macro defns conflict with xpdf's Object class
#ifdef LESSTIF_VERSION
#undef XtDisplay
#undef XtScreen
#undef XtWindow
#undef XtParent
#undef XtIsRealized
#endif

//------------------------------------------------------------------------

GString *XPDFCore::currentSelection = NULL;
XPDFCore *XPDFCore::currentSelectionOwner = NULL;
//Atom XPDFCore::targetsAtom;

//------------------------------------------------------------------------
// XPDFCoreTile
//------------------------------------------------------------------------

class XPDFCoreTile: public PDFCoreTile {
public:
  XPDFCoreTile(int xDestA, int yDestA);
  virtual ~XPDFCoreTile();
};

XPDFCoreTile::XPDFCoreTile(int xDestA, int yDestA):
  PDFCoreTile(xDestA, yDestA)
{
}

XPDFCoreTile::~XPDFCoreTile() 
{
}

//------------------------------------------------------------------------
// XPDFCore
//------------------------------------------------------------------------

XPDFCore::XPDFCore(SplashColorPtr paperColorA, Gulong paperPixelA,
		   Gulong mattePixelA, GBool reverseVideoA,
		   GBool installCmap, int rgbCubeSizeA):
  PDFCore(BTRON_COLORMODE, 4, reverseVideoA, paperColorA)
{
  GString *initialZoom;

  paperPixel = paperPixelA;
  mattePixel = mattePixelA;

  // get the initial zoom value
  initialZoom = globalParams->getInitialZoom();
  if (!initialZoom->cmp("page")) {
    zoom = zoomPage;
  } else if (!initialZoom->cmp("width")) {
    zoom = zoomWidth;
  } else {
    zoom = atoi(initialZoom->getCString());
    if (zoom <= 0) {
      zoom = defZoom;
    }
  }
  delete initialZoom;

//  panning = gFalse;

  // optional features default to on
  hyperlinksEnabled = gTrue;
  selectEnabled = gTrue;
}

XPDFCore::~XPDFCore() {
  if (currentSelectionOwner == this && currentSelection) {
    delete currentSelection;
    currentSelection = NULL;
    currentSelectionOwner = NULL;
  }
}

//------------------------------------------------------------------------
// loadFile / displayPage / displayDest
//------------------------------------------------------------------------

int XPDFCore::loadFile(GString *fileName, GString *ownerPassword,
		       GString *userPassword) 
{
  int err;

  err = PDFCore::loadFile(fileName, ownerPassword, userPassword);
  return err;
}

int XPDFCore::loadFile(LINK *link, GString *ownerPassword,
		       GString *userPassword) 
{
  int err;

  err = PDFCore::loadFile(link, ownerPassword, userPassword);
  return err;
}

int XPDFCore::loadFile(BaseStream *stream, GString *ownerPassword,
		       GString *userPassword) 
{
  return -1;
}

void XPDFCore::resizeToPage(int pg) 
{
//  printf("XPDFCore::resizeToPage: Unimplemented as of yet\n");
}

void XPDFCore::update(int topPageA, int scrollXA, int scrollYA,
		      double zoomA, int rotateA,
		      GBool force, GBool addToHist) 
{
  int oldPage;

  oldPage = topPage;
  PDFCore::update(topPageA, scrollXA, scrollYA, zoomA, rotateA,
		  force, addToHist);
//  linkAction = NULL;
#if 0
  if (doc && topPage != oldPage) {
    if (updateCbk) {
      (*updateCbk)(updateCbkData, NULL, topPage, -1, "");
    }
  }
#endif
}


//------------------------------------------------------------------------
// page/position changes
//------------------------------------------------------------------------

GBool XPDFCore::gotoNextPage(int inc, GBool top) {
  if (!PDFCore::gotoNextPage(inc, top)) {
    sig_buz(0);
    return gFalse;
  }
  return gTrue;
}

GBool XPDFCore::gotoPrevPage(int dec, GBool top, GBool bottom) {
  if (!PDFCore::gotoPrevPage(dec, top, bottom)) {
    sig_buz(0);
    return gFalse;
  }
  return gTrue;
}

GBool XPDFCore::goForward() 
{
  printf("XPDFCore::goForward: Unimplemented as of yet\n");
  return gFalse;
}

GBool XPDFCore::goBackward() 
{
  printf("XPDFCore::goBackward: Unimplemented as of yet\n");
  return gFalse;
}

//------------------------------------------------------------------------
// find
//------------------------------------------------------------------------

GBool XPDFCore::find(char *s, GBool caseSensitive,
		     GBool next, GBool backward, GBool onePageOnly) 
{
  printf("XPDFCore::find: Unimplemented as of yet\n");
  return gFalse;
}

GBool XPDFCore::findU(Unicode *u, int len, GBool caseSensitive,
		      GBool next, GBool backward, GBool onePageOnly) 
{
  printf("XPDFCore::findU: Unimplemented as of yet\n");
  return gFalse;
}

//------------------------------------------------------------------------
// misc access
//------------------------------------------------------------------------

void XPDFCore::setBusyCursor(GBool busy) 
{
  gset_ptr((busy ? PS_BUSY : PS_SELECT), NULL, -1, -1);
}

//------------------------------------------------------------------------
// GUI code
//------------------------------------------------------------------------

void XPDFCore::redrawRect(PDFCoreTile *tileA, int xSrc, int ySrc,
			  int xDest, int yDest, int width, int height) 
{
}

void XPDFCore::updateScrollbars() 
{
}

